@@ -1,6 +1,6 @@
 __title__ = 'datclass'
 __author__ = 'foyoux'
-__version__ = '0.0.14'
+__version__ = '0.0.18'
 __all__ = [
     'main',
     'DatClass',
@@ -17,8 +17,11 @@ __all__ = [
 ]
 
 import argparse
+import hashlib
 import json
+import keyword
 import os
+import string
 from dataclasses import dataclass, is_dataclass, field, asdict
 from pathlib import Path
 from typing import List, Dict, Union
@@ -28,8 +31,9 @@ try:
 except ImportError:
     from typing_extensions import get_origin, get_args, TypedDict
 
-_ORIGINAL_INIT = '__dataclass_init__'
 _DEBUG = False
+_NAME_MAP = {}
+_ORIGINAL_INIT = '__dataclass_init__'
 
 
 def set_debug(b):
@@ -37,12 +41,49 @@ def set_debug(b):
     _DEBUG = b
 
 
+def _get_md5_identifier(name, length=8):
+    s = hashlib.md5(name.encode()).hexdigest()
+    return f'a_{s[:length]}'  # attribute
+
+
+def get_ok_identifier(name: str):
+    # 查询缓存
+    if name in _NAME_MAP:
+        return _NAME_MAP[name]
+
+    # 如果是关键字，则加 '_' 后缀
+    if keyword.iskeyword(name):
+        s = f'{name}_'
+    elif name.isidentifier():
+        # 关键字是合法标识符，所以先判断关键字，再判断标识符
+        s = name
+    else:
+        # 不是标准标识符，过滤掉除 下划线、大小写字母、数字 的其他字符
+        s = ''.join(filter(lambda c: c in '_' + string.ascii_letters + string.digits, name))
+        if s:
+            if s[0] in string.digits:
+                s = f'a_{s}'
+            elif keyword.iskeyword(s):
+                s = f'{s}_'
+            elif not s.isidentifier():
+                s = _get_md5_identifier(name)
+        else:
+            s = _get_md5_identifier(name)
+
+    # 返回之前进行缓存
+    _NAME_MAP[name] = s
+    return s
+
+
 def _datclass_init(self, *args, **kwargs):
-    getattr(self, _ORIGINAL_INIT)(
-        *args, **{k: kwargs.pop(k) for k in self.__dataclass_fields__ if k in kwargs}
-    )
+    if kwargs:
+        kwargs = {get_ok_identifier(k): v for k, v in kwargs.items()}
     if _DEBUG:
-        return
+        getattr(self, _ORIGINAL_INIT)(*args, **kwargs)
+    else:
+        getattr(self, _ORIGINAL_INIT)(
+            *args, **{k: kwargs.pop(k) for k in self.__dataclass_fields__ if k in kwargs}
+        )
     for attr, value in kwargs.items():
         setattr(self, attr, value)
 
@@ -51,11 +92,10 @@ def _datclass_init(self, *args, **kwargs):
 class DatClass:
 
     def __new__(cls, *args, **kwargs):
-        obj = super().__new__(cls)
         if not hasattr(cls, _ORIGINAL_INIT):
             setattr(cls, _ORIGINAL_INIT, cls.__init__)
             setattr(cls, '__init__', _datclass_init)
-        return obj
+        return super().__new__(cls)
 
     def __post_init__(self, *args, **kwargs):
         for attr_name, FIELD in self.__dataclass_fields__.items():  # type: ignore
@@ -160,26 +200,28 @@ def gen_datclass(dat: Union[list, dict], name='Object', recursive=False, dict_=F
         imports.DatClass = True
         codes = ['@dataclass', f'class {name}(DatClass):']
 
-    for k, v in dat.items():
+    for k_, v in dat.items():
+        k = get_ok_identifier(k_)
+        c = '' if k == k_ else f'  # rename from \'{k_}\''
         v_t = get_v_type(v)
         v_d = get_t_default(v_t)
         t_s = get_t_string(v_t)
         if recursive and v and (isinstance(v, dict) or t_s == 'List[dict]'):
-            imports.List = True
             s = get_nice_cls_name(k)
             if isinstance(v, dict):
                 t_s = s
             elif t_s == 'List[dict]':
+                imports.List = True
                 t_s = f'List[{s}]'
             codes = gen_datclass(v, s, recursive=True, dict_=dict_) + ['', ''] + codes
         if t_s == 'Dict':
             imports.Dict = True
         if dict_:
-            codes.append(f'    {k}: {t_s}')
+            codes.append(f'    {k}: {t_s}{c}')
         else:
             if v_d.startswith('field'):
                 imports.field = True
-            codes.append(f'    {k}: {t_s} = {v_d}')
+            codes.append(f'    {k}: {t_s} = {v_d}{c}')
 
     return codes
 
@@ -202,9 +244,9 @@ def gen_typed_dict(dat: Union[list, dict], name='Object', recursive=False):
             elif t_s == 'List[dict]':
                 t_s = f'List[{s}]'
             codes = gen_typed_dict(v, s, recursive=True) + codes
-        if not imports.Dict and t_s == 'Dict':
+        if t_s == 'Dict':
             imports.Dict = True
-        if not imports.List and t_s.startswith('List'):
+        if t_s.startswith('List'):
             imports.List = True
         n_t_dict[k] = t_s
     s = ', '.join([f'\'{k}\': {v}' for k, v in n_t_dict.items()])
